(*

    Program: FRJ_SAP
    File:    DRAWING.PAS
    Copyright (C) 1995
      Francois Jullien <frj@ccr.jussieu.fr>
      Barry McMullin   <mcmullin@eeng.dcu.ie>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, in the file LICENSE.TXT; if not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

*)


(***************************************************************************)


(* DRAWING.PAS: module for graphical display handling. *)

unit Drawing;

interface
 uses
  Global, Graph, CRT;

 procedure SetStateWindow;
 procedure SetUnityWindow;
 procedure SetLogWindow;
 procedure GrWrite(s: string);
 procedure GrWriteLn(s: string);
 procedure Pause;
 procedure ConditionalPause;

 procedure init_dessin;

 procedure dessin;

 procedure DesUnit (iteration: integer; unite: boolean);

implementation
 (******************************************************)
 (************  CALCUL SUR IMAGES    *******************)
 (******************************************************)

 type

 rect =
   record
     x1: Integer;
     y1: Integer;
     x2: Integer;
     y2: Integer;
   end;

 var
  XWindo, YWindo, LonWindo, LarWindo, LonWindo1, LarWindo1: integer;
  X1Windo, Y1Windo, X2Windo, Y2Windo: integer;
  MinLogX, MinLogY, LogX, LogY: Integer;

 procedure init_dessin;
 var
  GraphDriver, GraphMode, GraphCode: Integer;

 begin
  XWindo := 30;
  Ywindo := 250;
  X1Windo := 50;
  Y1Windo := 30;
  X2Windo := 300;
  Y2Windo := 30;
  LonWindo := 200;
  LarWindo := 200;
  LonWindo1 := 500;
  LarWindo1 := 200;

  GraphDriver := Detect;
  InitGraph(GraphDriver,GraphMode,'');
  GraphCode := GraphResult;
  if GraphCode <> grOK
  then
  begin
    WriteLn('Graphics error: ',
      GraphErrorMsg(GraphCode));
    Halt(1);
  end;

  MoveTo(X1Windo, Y1Windo - 12);
  OutText('State');
  rectangle(X1Windo-2, Y1Windo-2,
    X1Windo+LonWindo+2, Y1Windo+LarWindo+2);

  MoveTo(X2Windo, Y2Windo - 12);
  OutText('Unity');
  rectangle(X2Windo-2, Y2Windo-2,
    X2Windo+LonWindo+2, Y2Windo+LarWindo+2);

  MoveTo(XWindo, YWindo - 12);
  OutText('Logging');
  rectangle(XWindo-2, YWindo-2,
    XWindo+LonWindo1+2, YWindo+LarWindo1+2);

  MinLogX := 2;
  MinLogY := 2;
  LogX := MinLogX;
  LogY := MinLogY;

 end;

 procedure SetStateWindow;
 begin
  SetViewPort(X1Windo, Y1Windo,
   X1Windo+LonWindo, Y1Windo+LarWindo, true);
 end;

 procedure SetUnityWindow;
 begin
  SetViewPort(X2Windo, Y2Windo,
   X2Windo+LonWindo, Y2Windo+LarWindo, true);
 end;

 procedure SetLogWindow;
 begin
  SetViewPort(XWindo, YWindo,
   XWindo+LonWindo1, YWindo+LarWindo1, true);
  SetColor(White);
 end;

 procedure GrWrite(s: string);
 begin
  SetLogWindow;
  MoveTo(LogX, LogY);
  OutText(s);
  LogX := GetX;
  LogY := GetY;
 end;

 procedure GrWriteLn(s: string);
 begin
  GrWrite(s);
  LogX := MinLogX;
  LogY := LogY + 10;
  if LogY > (LarWindo1 - 10)
  then
   LogY := MinLogY;
  SetFillStyle(SolidFill,Black);
  bar(LogX, LogY, LonWindo1, LogY+9);
 end;

 procedure Pause;
 var
  Ch: Char;
 begin
   if (Readkey = #0)
   then
    Ch := ReadKey;
 end;

 procedure ConditionalPause;
 begin
   if KeyPressed
   then
   begin
    Pause; (* Get first keypress ... *)
    Pause; (* ... then wait for another. *)
   end;
 end;

 procedure SetRect(VAR r: rect;
   x1: Integer; y1: Integer;
   x2: Integer; y2: Integer);
 begin
   r.x1 := x1;
   r.y1 := y1;
   r.x2 := x2;
   r.y2 := y2;
 end;

 procedure EraseRect(r: rect);
 begin
  SetFillStyle(EmptyFill,Black);
  bar(r.x1, r.y1, r.x2, r.y2);
 end;

 procedure FrameRect(r: rect);
 begin
  rectangle(r.x1, r.y1, r.x2, r.y2);
 end;

 procedure PaintRect(r: rect);
 begin
  SetFillStyle(SolidFill,Yellow);
  bar(r.x1, r.y1, r.x2, r.y2);
 end;

 procedure dessin;
  var
   r: rect;
   i, j, k, l: integer;

   xi, lon, lon2, yi, lar, lar2, DiagLon, DiagLar: integer;

 begin

  SetStateWindow;
  SetColor(LightRed);
(*  penpat(Black);*)

  lon := trunc(LonWindo * 0.5 / n);
  lon2 := lon * 2;
  lar := trunc(LarWindo * 0.5 / n);
  lar2 := lar * 2;
  DiagLar := trunc(sqrt(2) * lar2);
  DiagLon := trunc(sqrt(2) * lon2);
  for i := 1 to n do
   begin
    for j := 1 to n do
     begin
      xi := trunc((i - 1) * lon2);
      yi := trunc((j - 1) * lar2);
      SetRect(r, xi, yi, xi + lon2, yi + lar2);
      EraseRect(r);
      SetRect(r, xi, yi, xi + lon, yi + lar);
      if esp[i, j].el = 0 then
       FrameRect(r);
      if esp[i, j].el = 2 then
       PaintRect(r);
      if esp[i, j].el > 2 then
       begin
        SetColor(LightBlue);
        FrameRect(r);
        if esp[i, j].el > 3 then
         begin
          xi := xi + lon div 2;
          yi := yi + lar div 2;
          l := 0;
          while
           esp[i, j].lien^[tt_Vois[l][1], tt_Vois[l][2]] = false
          do l := l + 1;
          Moveto(xi, yi);
          if l div 2 = 0 then
           LineTo(xi + lon2 * tt_Vois[l][1], yi + lar2 * tt_Vois[l][2])
          else
           LineTo(xi + lon2 * tt_Vois[l][1], yi + lar2 * tt_Vois[l][2]);
          if esp[i, j].el > 4 then
           begin
            l := l + 1;
            while
             esp[i, j].lien^[tt_Vois[l][1], tt_Vois[l][2]] = false
            do l := l + 1;
            if l > 7 then
             readln;
            Moveto(xi, yi);
            if l div 2 = 0 then
             LineTo(xi + lon2 * tt_Vois[l][1], yi + lar2 * tt_Vois[l][2])
            else
             LineTo(xi + lon2 * tt_Vois[l][1], yi + lar2 * tt_Vois[l][2])
           end;
         end;
        SetColor(LightRed);

       end;
     end;
   end;
 end;

 procedure DesUnit;
  var
   i: integer;
 begin
  SetUnityWindow;
  SetColor(LightGreen);
  if unite = true then
   i := 1
  else
   i := 2;
  MoveTo(trunc((iteration / niter) * LonWindo),
   round(i * LarWindo / 3));
  LineTo(GetX+1, GetY);

  (*
   MoveTo(iteration, round(i * LarWindo1 / 3));
   Line(0, 0);
  *)
 end;

end.
