(*

    Program: FRJ_SAP
    File:    FRJ_SAP.PAS
    Copyright (C) 1995
      Francois Jullien <frj@ccr.jussieu.fr>
      Barry McMullin   <mcmullin@eeng.dcu.ie>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, in the file LICENSE.TXT; if not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

*)


(*********************)

(* FRJ_SAP.PAS: Main program module. *)

program auto;

 uses
  Global, Drawing, Graph;

 (**********************************************************************)
 (*____________________PROCEDURES DE SORTIE____________________________*)
 (**********************************************************************)
 procedure afmat;
  var
   i, j: integer;
 begin
  for i := 1 to n do
   begin
    GrWriteln(' ');
    for j := 1 to n do
    begin
     Str(esp[i, j].el : 3,s);
     GrWrite(s);
     end;
   end;
  GrWriteln(' ');
 end;

 procedure liliste (tete: noeud);
  var
   indic: ptnoeud;
 begin
  indic := tete.d;
  while indic <> nil do
   begin
    Str(indic^.coo[1], s);
    GrWrite(s);
    GrWrite(' ');
    Str(indic^.coo[2], s);
    GrWrite(s);
    GrWrite(' ');
    (* write(indic^.coo[1], ' ', indic^.coo[2], '  ');*)
    indic := indic^.d;
   end;
  GrWriteln(' ');
 end;

 (***********************************************************************)
 (*____________________PROCEDURES D'INITIALISATION______________________*)
 (***********************************************************************)
 procedure Cre_List (var head: tetes);
  var
   i, j: integer;
   X: Coord_Abs;
   Mark: array[1..5] of ptnoeud;
 begin
  for i := 1 to 5 do
   Mark[i] := @head[i];
  for i := 0 to 7 do
   for j := 1 to 2 do
    vois_card[i, j] := 0;
  vois_card[0, 1] := 1;
  vois_card[1, 2] := 1;
  vois_card[2, 1] := -1;
  vois_card[3, 2] := -1;
  tt_vois[0, 1] := 1;
  tt_vois[0, 2] := 0;
  tt_vois[1, 1] := 1;
  tt_vois[1, 2] := 1;
  tt_vois[2, 1] := 0;
  tt_vois[2, 2] := 1;
  tt_vois[3, 1] := -1;
  tt_vois[3, 2] := 1;
  tt_vois[4, 1] := -1;
  tt_vois[4, 2] := 0;
  tt_vois[5, 1] := -1;
  tt_vois[5, 2] := -1;
  tt_vois[6, 1] := 0;
  tt_vois[6, 2] := -1;
  tt_vois[7, 1] := 1;
  tt_vois[7, 2] := -1;

  for i := 1 to n do
   for j := 1 to n do
    begin
     if Esp[i, j].el <> 0 then
      begin
       X[1] := i;
       X[2] := j;
       new(mark[Esp[i, j].el]^.d);
       mark[Esp[i, j].el]^.d^.g := mark[Esp[i, j].el];
       mark[Esp[i, j].el] := mark[Esp[i, j].el]^.d;
       mark[Esp[i, j].el]^.coo := X;
       mark[Esp[i, j].el]^.d := nil;
      end;
    end;
 end;

 procedure initialise;
  var
   i, j: integer;
 begin

  for i := 1 to 5 do
   head[i].d := nil;

  for i := 1 to n do
   for j := 1 to n do
    Esp[i, j].el := 0;
  Esp[8, 10].el := 2;
  Esp[15, 15].el := 2;
  Esp[2, 3].el := 1;
  Esp[7, 2].el := 1;
  Esp[2, 6].el := 1;
  Esp[7, 3].el := 1;
  Esp[3, 4].el := 3;
  Esp[7, 4].el := 1;
  Esp[8, 6].el := 1;
  Esp[7, 8].el := 1;
  Esp[9, 6].el := 3;
  cre_list(head);
 end;

 procedure Init (var X: Pttab);
  var
   i, j: integer;
 begin
  new(X);
  for i := -1 to 1 do
   for j := -1 to 1 do
    X^[i, j] := false;
 end;

 (***********************************************************************)
 (*____________________PROCEDURES DE MANIPULATION DES LISTES____________*)
 (***********************************************************************)
 function Test_fin (indic: ptnoeud): boolean;
 begin
  if indic^.d = nil then
   Test_fin := true
  else
   Test_fin := false;
 end;

 procedure insere (var tete: noeud; X: ptnoeud);
 begin
  X^.d := tete.d;
  X^.g := @tete;
  if Test_fin(X) = false then
   tete.d^.g := X;
  tete.d := X;
 end;

 procedure recherche (var indic: ptnoeud; Poin: Coord_Abs);
 begin
  indic := head[Esp[Poin[1], Poin[2]].el].d;
  while (indic^.coo[1] <> Poin[1]) or (indic^.coo[2] <> Poin[2]) do
   indic := indic^.d;
 end;



 procedure supprime (var indic: ptnoeud);
 begin
  if Test_fin(indic) = false then
   begin
    indic^.g^.d := indic^.d;
    indic^.d^.g := indic^.g;
    dispose(indic);
    indic := nil;
   end
  else
   begin
    indic^.g^.d := nil;
    dispose(indic);
    indic := nil;
   end;
 end;

 procedure echange (var indic: ptnoeud; var tete: noeud);
 begin
  indic^.g^.d := indic^.d;
  if Test_fin(indic) = false then
   indic^.d^.g := indic^.g;
  indic^.d := tete.d;
  indic^.g := @tete;
  if tete.d <> nil then
   tete.d^.g := indic;
  tete.d := indic;
 end;

 (************************************************************************)
 (*____________________PROCEDURES DE PERMUTATION_________________________*)
 (************************************************************************)

  (* permet la permutation circulaire de un a quatre element: il
     faut mettre a jour l 'epace et les listes *)

 procedure actualiste (A, B: Coord_Abs);
  var
   indic: ptnoeud;
 begin
  if Esp[A[1], A[2]].el <> 0 then
   begin
    recherche(indic, A);
    indic^.coo := B;
   end;
 end;

 procedure Permutte_2 (X, Y: Coord_abs);
  var
   muet: point;
 begin
  actualiste(X, Y);
  actualiste(Y, X);
  muet := Esp[X[1], X[2]];
  Esp[X[1], X[2]] := Esp[Y[1], Y[2]];
  Esp[Y[1], Y[2]] := muet;
 end;

 procedure Permutte_3 (X, Y, Z: Coord_abs);
  var
   muet: point;
 begin
  actualiste(X, Y);
  actualiste(Y, Z);
  actualiste(Z, X);
  muet := Esp[X[1], X[2]];
  Esp[X[1], X[2]] := Esp[Z[1], Z[2]];
  Esp[Z[1], Z[2]] := Esp[Y[1], Y[2]];
  Esp[Y[1], Y[2]] := muet;
 end;

 procedure Permutte_4 (W, X, Y, Z: Coord_abs);
  var
   muet: point;
 begin
  actualiste(W, X);
  actualiste(X, Y);
  actualiste(Y, Z);
  actualiste(Z, W);
  muet := Esp[W[1], W[2]];
  Esp[W[1], W[2]] := Esp[Z[1], Z[2]];
  Esp[Z[1], Z[2]] := Esp[Y[1], Y[2]];
  Esp[Y[1], Y[2]] := Esp[X[1], X[2]];
  Esp[X[1], X[2]] := muet;
 end;

 procedure Permutte (W, X, Y, Z: Coord_abs);
 begin
  if (W[1] = Y[1]) and (W[2] = Y[2]) then
   Permutte_2(W, Z)
  else if (W[1] = X[1]) and (W[2] = X[2]) then
   Permutte_3(X, Y, Z)
  else
   Permutte_4(W, X, Y, Z);
 end;

 (************************************************************************)
 (*____________PETITES PROCEDURES DE BASES_______________________________*)
 (************************************************************************)
 function Test_Space (x, y: integer): boolean;
 begin
  if (x < 1) or (x > n) or (y < 1) or (y > n) then
   Test_Space := false
  else
   Test_Space := true;
 end;

 function Nul (U: Coord_Rel): boolean;
 begin
  if (U[1] = 0) and (U[2] = 0) then
   Nul := true
  else
   Nul := false;
 end;

 procedure Enleve_Vois (var m: Nbr_Vois; var U: Coord_Rel);
 begin
  U[1] := 0;
  U[2] := 0;
  m := m - 1;
 end;

 procedure Nv_Vois (var m: Nbr_Vois; U: Coord_Rel; var stok: Tab_Vois);
 begin
  stok[m][1] := U[1];
  stok[m][2] := U[2];
  m := m + 1;
 end;

 function myrand (l: integer): integer;
 begin
    myrand := Random(l);
    (* myrand := trunc((Random / 32767 + 1) * l * 0.5); *)
      (* renvoie un entier aleatoire entre 0 et l-1*)
 end;

 procedure Choisi (
    m: Nbr_Vois;
    X: Coord_Abs;
    var Y: Coord_Abs;
    stok: Tab_Vois);

  var
   alea: Nbr_Vois;
 begin
  alea := myrand(m);
  Y[1] := stok[alea][1] + X[1];
  Y[2] := stok[alea][2] + X[2];
 end;

 procedure selection (
    X: Coord_abs;
    var Y: Coord_abs;
    var echoue: boolean);
  (* choisi de maniere aleatoire un voisin cardinal
     quelconque Y de X *)
  (* choose in an aleatory way a cardinal neighbour Y of X *)
  var
   m: Nbr_Vois;
   stok: Tab_Vois;
   i: integer;
 begin
  echoue := false;
    if (X[1] > 1) and (X[1] < n) and (X[2] > 1) and (X[2] < n) then
      (*X not in the border*)
     Choisi(4, X, Y, vois_card)
  else
   begin      (* X in the border of the space*)
    for i := 0 to 7 do
     begin
      stok[i][1] := 0;
      stok[i][2] := 0;
     end;
    m := 0;
    for i := 0 to 3 do
     begin
      if Test_Space(X[1] + vois_card[i][1], X[2] + vois_card[i][2]) = true
       then
        Nv_Vois(m, vois_card[i], stok);
     end;
    Choisi(4, X, Y, stok);
    if (X[1] = Y[1]) and (X[2] = Y[2]) then
     echoue := true;
   end;
 end;

 procedure cher_ele_vrac (
    X: Coord_abs;
    var Y: Coord_abs;
    j: sorte;
    var echoue: boolean);

  var
   U: Coord_Rel;
   m: Nbr_vois;
   stok: Tab_Vois;
   i, k, l: integer;
 begin
  echoue := false;
  m := 0;
  for i := 0 to 7 do
   begin
    k := X[1] + tt_vois[i][1];
    l := X[2] + tt_vois[i][2];
    if (Test_Space(k, l) = true) and (Esp[k, l].el = j) then
     Nv_Vois(m, tt_vois[i], stok);
   end;
  if m <> 0 then
   Choisi(m, X, Y, stok)
  else
   echoue := true;
 end;

 procedure cherlien (
    X: Coord_abs;
    var Y: Coord_abs;
    var echoue: boolean);

  var
   i, k, l: integer;
   V: Coord_Rel;
   m: Nbr_Vois;
   stok: Tab_Vois;
 begin
  echoue := false;
  m := 0;
  for i := 0 to 3 do
   begin
    k := X[1] + 2 * vois_card[i][1];
    l := X[2] + 2 * vois_card[i][2];
    if (Test_Space(k, l) = true)
     and (Esp[k, l].el = 1)
     and (Esp[X[1] + vois_card[i][1], X[2] + vois_card[i][2]].el >= 4)
     then
      begin
       V[1] := 2 * vois_card[i][1];
       V[2] := 2 * vois_card[i][2];
       Nv_Vois(m, V, stok);
      end;
   end;
  if m <> 0 then
   Choisi(m, X, Y, stok)
  else
   echoue := true;
 end;

 procedure cher_ele (
    X: Coord_abs;
    j: sorte;
    var stok:
    Tab_Vois;
    var m: Nbr_Vois);

  (* cherche, parmi les voisins de X, ceux de l'espece j, et les
     stocke sous forme de coordonnees relatives, dans stok; m est
     le nombre de tels voisins *)
  (* look for , among the neighbours of X , those of the species
     j , and stock them with their relativ coordinates in stock;
     m is the number of such neighbours *)

  var
   k, l: integer;
   i, o: Nbr_Vois;
 begin
  for i := 0 to 7 do
   begin
    k := X[1] + tt_vois[i][1];
    l := X[2] + tt_vois[i][2];
    if (Test_Space(k, l) = true) and (Esp[k, l].el = j) then
     begin
      o := i;
      Nv_Vois(o, tt_vois[i], stok);
      m := m + 1;
     end;
   end;
 end;

 (*********************************************************************)
 (*____________PROCEDURES  DE LIAISON_________________________________*)
 (*********************************************************************)
 procedure Select_Angle (
    indic: ptnoeud;
    var stok: Tab_Vois;
    var m: Nbr_Vois);

  (* parmi les candidats a la liaison qui sont deja lies, ne
     conserve que ceux ayant un angle correct *)

  (* among the candidates to be bounded which are already
     bounded, keep only those which have a correct angle*)

  var
   x, y: dim;
   i: integer;

 begin
  x := indic^.coo[1];
  y := indic^.coo[2];
  for i := 0 to 7 do
   begin
    if (Nul(stok[i]) = false)
     and (Esp[x + stok[i][1], y + stok[i][2]].el > 3)
     then
      begin
       if i mod 2 <> 0 then  (* coin *)
        begin
         if (Esp[x + stok[i][1], y + stok[i][2]].lien^
           [(-stok[i][1] + 1) mod 2, -stok[i][2]] = true)
          or (Esp[x + stok[i][1], y + stok[i][2]].lien^
           [-stok[i][1], (-stok[i][2] + 1) mod 2] = true)
          then
           enleve_vois(m, stok[i]);
        end
       else if i mod 4 = 0 then  (*verticale*)
        begin
         if (Esp[x + stok[i][1], y].lien^[-stok[i][1], 1] = true)
          or (Esp[x + stok[i][1], y].lien^[-stok[i][1], -1] = true)
          then
           enleve_vois(m, stok[i]);
        end
       else
        begin   (*horizontale*)
         if (Esp[x, y + stok[i][2]].lien^[-1, -stok[i][2]] = true)
          or (Esp[x, y + stok[i][2]].lien^[1, -stok[i][2]] = true)
          then
           enleve_vois(m, stok[i]);
        end;
      end;
   end;
 end;

 procedure lie1 (
    var k, l: integer;
    alea: integer;
    x, y: dim;
    stok: Tab_Vois;
    j: sorte);

  (*essaie de former une liaison*)
  (*try to make a link*)
  var
   Z, indic: ptnoeud;
 begin
  new(Z);
  repeat
   k := k + 1;
   if Nul(stok[k]) = false then
    l := l + 1;
  until l = alea;
  Esp[x, y].lien^[stok[k][1], stok[k][2]] := true;
  Z^.coo[1] := x + stok[k][1];
  Z^.coo[2] := y + stok[k][2];
  recherche(indic, Z^.coo);
  if j = 4 then
   init(Esp[indic^.coo[1], indic^.coo[2]].lien);
  Esp[indic^.coo[1], indic^.coo[2]].el := j;
  Esp[indic^.coo[1], indic^.coo[2]].lien^[-stok[k][1], -stok[k][2]] := true;
  echange(indic, head[j]);
  dispose(Z);
 end;

 procedure lie2 (
    x, y: dim;
    indic: ptnoeud;
    var m: Nbr_vois;
    k: integer;
    var stok: Tab_Vois);

  (* essaie de former une 2ieme liaison avec un M libre *)
  (* try to make a second link with a free M *)
  var
   o, p: integer;
   alea: integer;

 begin
    (* enleve de la liste des M libres ceux aux
       angles incompatibles *)
    (* remove from the list of the free M those with not
       compatible angles*)
  if Nul(stok[(k + 1) mod 8]) = false then
   Enleve_Vois(m, stok[(k + 1) mod 8]);
  if Nul(stok[(k + 7) mod 8]) = false then
   Enleve_Vois(m, stok[(k + 7) mod 8]);

    (*si il en reste,en choisi un,le lie,et met a jour les listes*)
    (*if their remain some, choose onen, and refresh the lists*)
  if m <> 0 then
   begin
    alea := trunc((Random / 32767 + 1) * m * 0.5);
    o := -1;
    p := -1;
    lie1(o, p, alea, x, y, stok, 4);
    Esp[x, y].el := 5;
    echange(indic, head[5])
   end
      (*sinon met a jour la liste*)
      (*otherwise refresh the list*)
  else
   begin
    GrWriteln('APPEL A LIE 2 SANS REUSSITE DE LIAISON');
    Esp[x, y].el := 4;
    echange(indic, head[4]);
   end;
 end;

 procedure enleve_contigu (var stok: Tab_vois; var m: Nbr_Vois);
  var
   i, j: integer;
   p, o: Nbr_Vois;
   temp, tempo: array[0..7] of boolean;
 begin
  for i := 0 to 1 do
   begin
    for j := 0 to 7 do
     temp[j] := false;
    if nul(stok[i]) = false then
     begin
      temp[i] := true;
      o := 1;
     end
    else
     o := 0;
    for j := i + 1 to 7 do
     begin
      if (Nul(stok[j]) = false)
       and (temp[j - 1] = false)
       and (temp[(j + 1) mod 8] = false)
       then
       begin
        temp[j] := true;
        o := o + 1;
       end;
     end;
    if i = 0 then
     begin
      p := o;
      tempo := temp;
     end;
   end;
  if p > o then
   begin
    m := p;
    for i := 0 to 7 do
     if tempo[i] = false then
      begin
       stok[i][1] := 0;
       stok[i][2] := 0;
      end;
   end
  else
   begin
    m := o;
    for i := 0 to 7 do
     if temp[i] = false then
      begin
       stok[i][1] := 0;
       stok[i][2] := 0;
      end;
   end;
 end;

 procedure liaison (X: ptnoeud);
  (* cherche a relier le maillon X *)
  (* try to link the X*)
  var
   voisml: Tab_Vois;
        (*  tableaux des voisins relatifs maillon lie une fois de X *)
        (*  array of the relatives neighbours of X linked one time *)
   voism: Tab_Vois;
        (*  tableaux des voisins relatifs mailon libre de X  *)
        (*  array of the relatives neighbours of X which are free *)
   ml, m: Nbr_Vois;
   alea, alea1, k, l: integer;
   t, u: dim;
 begin
  for k := 0 to 7 do
   begin
    voisml[k][1] := 0;
    voisml[k][2] := 0;
    voism[k][1] := 0;
    voism[k][2] := 0;
   end;
  t := X^.coo[1];
  u := X^.coo[2];
  ml := 0;
  Cher_Ele(X^.coo, 4, voisml, ml);
     (* recherche des maillons lies une fois contigus a X *)
     (* look for the elemlent linked one time and which touch X *)
  Select_Angle(X, voisml, ml);
     (* enleve ceux dont la liaison avec X donnerait un angle
        inferieur a 90 degre *)
     (* remove those whom link with X would give an angle
        less than 90 degre *)
  enleve_contigu(voisml, ml);
  k := -1;
  l := -1;
  if ml >= 2 then
   begin
        (* on peut lier X a deux maillons deja lie chacun une
           fois *)
        (* we can linked X to two elements already linked
           each one one time*)
    repeat
     alea := myrand(ml);
     alea1 := myrand(ml);
    until alea < alea1;
    Init(Esp[t, u].lien);
     (* cre un tableau de liaison pour X *)
     (* create an array of linkage forX *)
    lie1(k, l, alea, t, u, voisml, 5);
     (* effectue la premiere liaison *)
     (* make the first link *)
    lie1(k, l, alea1, t, u, voisml, 5);(* la deuxieme *)
     (* the second *)
    Esp[t, u].el := 5;
    GrWrite('1liais ');
    Str(Esp[t, u].el : 3, s);
    GrWrite(s);
    GrWrite(' ');
    Str(t : 3, s);
    GrWrite(s);
    GrWrite(' ');
    Str(u : 3, s);
    GrWriteLn(s);
    (* writeln('1liais ', Esp[t, u].el : 3, ' ', t : 3, ' ', u : 3); *)
    echange(X, head[5]);
   end
  else
   begin
    m := 0;
    Cher_Ele(X^.coo, 3, voism, m);
          (* recherche des maillons libres contigus a X *)
          (* look for free elements which touch X *)
    if ml = 1 then
          (* on peut lier X a un maillon deja lie une fois *)
          (* we can link X to an element already linde one time *)
     begin
      Init(Esp[t, u].lien);
      alea := 0;
      lie1(k, l, alea, t, u, voisml, 5);
      Esp[t, u].el := 4;
      GrWrite('2liais ');
      Str(Esp[t, u].el : 3, s);
      GrWrite(s);
      GrWrite(' ');
      Str(t : 3, s);
      GrWrite(s);
      GrWrite(' ');
      Str(u : 3, s);
      GrWriteLn(s);
      (* writeln('2liais', Esp[t, u].el : 3, ' ', t : 3, ' ', u : 3); *)
      echange(X, head[4]);
       if m >= 1 then
              (* on peut lier X a un maillon libre *)
              (* we can link X to an element free *)
        begin
        lie2(t, u, X, m, k, voism);
                (* Esp[t, u].el := 5; *)
        GrWrite('3liais ');
        Str(Esp[t, u].el : 3, s);
        GrWrite(s);
        GrWrite(' ');
        Str(t : 3, s);
        GrWrite(s);
        GrWrite(' ');
        Str(u : 3, s);
        GrWriteLn(s);
        writeln('3liais', Esp[t, u].el : 3, ' ', t : 3, ' ', u : 3);
                (* echange(X, head[5]); *)
        end;
     end
    else
          (* on ne peut lier X a un maillon deja lie une fois *)
          (* we can not link X to an element already linked one time*)

     begin
      if m >= 1 then
       begin
        alea := myrand(m);
        Init(Esp[t, u].lien);
        lie1(k, l, alea, t, u, voism, 4);
        Esp[t, u].el := 4;
        GrWrite('4liais ');
        Str(Esp[t, u].el : 3, s);
        GrWrite(s);
        GrWrite(' ');
        Str(t : 3, s);
        GrWrite(s);
        GrWrite(' ');
        Str(u : 3, s);
        GrWriteLn(s);
        (* writeln('4liais', Esp[t, u].el : 3, ' ', t : 3, ' ', u : 3);*)
        echange(X, head[4]);
        Enleve_Vois(m, voism[k]);
        if m >= 1 then
         begin
          lie2(t, u, X, m, k, voism);
                    (* Esp[t, u].el := 5; *)
                    (* echange(X, head[5]); *)
          GrWrite('5liais ');
          Str(Esp[t, u].el : 3, s);
          GrWrite(s);
          GrWrite(' ');
          Str(t : 3, s);
          GrWrite(s);
          GrWrite(' ');
          Str(u : 3, s);
          GrWriteLn(s);
          writeln('5liais', Esp[t, u].el : 3, ' ', t : 3, ' ', u : 3);
         end;
       end;
     end;
   end;
 end;

 (********************************************************************)
 (*____________PROCEDURES DE MOUVEMENT    ___________________________*)
 (********************************************************************)
 procedure Bouge_S (X, Y, S: Coord_abs);
  var
   Z: Coord_abs;
   echoue: boolean;
 begin
  cher_ele_vrac(S, Z, 1, echoue);
  if echoue = false then
   permutte(X, Y, S, Z)
  else
   begin
    cherlien(S, Z, echoue);
    if echoue = false then
     permutte(X, Y, S, Z)
    else
     permutte(X, X, Y, S);
   end;
 end;

 procedure Bouge_M (M: Coord_abs); (*bouge means move*)
  var
   Y: Coord_abs;
   echoue: boolean;  (*echoue means fail*)
   indic, mark: ptnoeud;
 begin
  selection(M, Y, echoue);
  if (echoue = false) and (Esp[Y[1], Y[2]].el < 2) then
      (* le voisin Y de M est un trou ou un Substrat *)
      (* the neighbour Y of M is a whole or a substrat*)
   begin
    if Esp[Y[1], Y[2]].el = 1 then
          (*le voisin de M est un trou*)
     permutte(M, M, M, Y)
            (* deplace le Maillon dans le trou *)
            (* put M in the Whole *)
    else
     Bouge_S(M, M, Y);
            (* le voisin de M est un Substrat *)
            (* neighbour of M is a Substrat *)
    recherche(mark, Y);
    liaison(mark);
   end
 end;

 procedure Bouge_CM (C, M: Coord_abs);
  var
   Y: Coord_abs;
   echoue: boolean;
   indic, mark: ptnoeud;
 begin
  cher_ele_vrac(M, Y, 1, echoue);
    if (echoue = false) then
      (* neighbour is a whole *)
   begin
    permutte(C, C, M, Y);
    recherche(mark, Y);
    liaison(mark);
   end
  else
   begin
    cher_ele_vrac(M, Y, 0, echoue);
    if echoue = false then
     begin
      Bouge_S(C, M, Y);
              (*vois S*)
              (*lie M*)
      recherche(mark, Y);
      liaison(mark);
     end
    else
     begin
      permutte(C, C, C, M);
            (* lie M *)
      ;
      recherche(mark, C);
      liaison(mark);
     end;
   end;
 end;

 procedure Move_vide;
  var
   X, Y: Coord_abs;
   echoue: boolean;
   indic, mark: ptnoeud;
   k, l, i: integer;
 begin
  indic := head[1].d;
  while indic <> nil do
   begin
    selection(indic^.coo, X, echoue);
    if (echoue = false) and (Esp[X[1], X[2]].el <> 1) then
          (*neighbour not a whole*)
     begin
      if Esp[X[1], X[2]].el < 4 then
              (*neighbour not linked*)
       begin
        Y := indic^.coo;
        permutte(X, X, X, indic^.coo);
        if Esp[Y[1], Y[2]].el = 3 then
         begin
          recherche(mark, Y);
          liaison(mark);
         end;
       end
      else
       begin
        k := 2 * X[1] - indic^.coo[1];   (*vois lie*)
        l := 2 * X[2] - indic^.coo[2];
        if (Test_Space(k, l) = true) and (Esp[k, l].el = 0)
         then
         begin
          Y[1] := k;
          Y[2] := l;
          permutte(Y, Y, Y, indic^.coo);
         end;
       end;
     end;
    indic := indic^.d;
   end;
 end;

 procedure Move_Maillon;
  var
   indic: ptnoeud;
   i: integer;
   nw: integer;
   Tw: array[1..100] of Coord_Abs;
 begin
  nw := 0;
  for i := 1 to 100 do
   begin
    Tw[i][1] := 0;
    Tw[i][2] := 0;
   end;
  indic := head[3].d;
  while indic <> nil do
   begin
    i := 1;
    while (i < nw + 1)
     and ((indic^.coo[1] <> Tw[i][1]) or (indic^.coo[2] <> Tw[i][2]))
     do
      i := i + 1;
    if i = (nW + 1) then
     begin
      nw := nw + 1;
      bouge_M(indic^.coo);
      Tw[nw] := indic^.coo;
      indic := head[3].d;
     end
    else
     indic := indic^.d;
   end;
 end;

 procedure Move_Cata;
  var
   X, Y: Coord_abs;
   echoue: boolean;
   indic: ptnoeud;
 begin
  indic := head[2].d;
  while indic <> nil do
   begin
    selection(indic^.coo, X, echoue);
    if (echoue = false)
     and (Esp[X[1], X[2]].el <= 3)
     and (Esp[X[1], X[2]].el <> 2)
     then (*voisin pas cata,ml*)
     begin
      if Esp[X[1], X[2]].el = 1 then
              (* le voisin du Catalyseur est un trou *)
              (* the neighbour of the catalyst is a whole *)
       permutte(X, X, X, indic^.coo)
      else if Esp[X[1], X[2]].el = 0 then
              (* le voisin du Catalyseur est un Substrat *)
              (* the neighbour of the catalyst is a substrat*)

       bouge_S(indic^.coo, indic^.coo, X)
      else
       bouge_CM(indic^.coo, X);
                (*le voisin du Catalyeur est un maillon libre*)
                (* the neighbour of the catalyst is a free element*)

     end;
    indic := indic^.d;
   end;
 end;

 (****************************************************************)
 (*____________PROCEDURES DE PRODUCTION   _______________________*)
 (****************************************************************)
 procedure Enleve_S (X: ptnoeud; U: Coord_Rel; j: sorte);
  (*enleve du type substrat le voisin de X de coordonnees
    relatives U et l'insere dans le type j*)

  (*remove of the type substrat the neighbour of X of relatives
    coordinates U and insert it in the type j*)

  var
   Y: ptnoeud;
 begin
  new(Y);
  Y^.coo[1] := X^.coo[1] + U[1];
  Y^.coo[2] := X^.coo[2] + U[2];
  Esp[Y^.coo[1], Y^.coo[2]].el := j;
  insere(head[j], Y);
  if j = 3 then
   begin
    GrWrite('pro ');
    Str(Y^.coo[1] : 3, s);
    GrWrite(s);
    GrWrite(' ');
    Str(Y^.coo[2] : 3, s);
    GrWriteLn(s);
    (* writeln('pro ', Y^.coo[1] : 3, ' ', Y^.coo[2]);*)
    liaison(Y);
   end;
  Y := nil;
 end;

 procedure chercontigu (var stok: Tab_Vois; var m: Nbr_Vois);
  var
   i: integer;
 begin
  for i := 0 to 7 do
   begin
    if (Nul(stok[i]) = false)
     and (Nul(stok[(i + 1) mod 8]) = true)
     and (Nul(stok[(i + 7) mod 8]) = true)
     then
      Enleve_Vois(m, stok[i]);
   end;
 end;

 procedure Production;
  var
   indic: ptnoeud;
   m, alea: Nbr_vois;
   stok: Tab_Vois;
   k, l: integer;
 begin
  indic := head[2].d;
  while indic <> nil do
   begin
    for k := 0 to 7 do
     begin
      stok[k][1] := 0;
      stok[k][2] := 0;
     end;
    m := 0;
    cher_ele(indic^.coo, 0, stok, m);
          (* cherche les vois du cata qui sont des substrats *)
          (*look for the neighbour of the catalyst which are substrates*)
    if m <> 0 then
     begin
      chercontigu(stok, m);
              (* enleve ceux qui sont isoles *)
              (*remove those which are isolated*)
      if m <> 0 then
       begin
        alea := myrand(m);
        k := -1;
        l := -1;
        repeat
         k := k + 1;
         if Nul(stok[k]) = false then
          l := l + 1;
        until l = alea;
                (*le nouv maillon*)
                (*the new element*)
        enleve_S(indic, stok[k], 3);
                (* l'autre S a diparaitre *)
                (* the other S to desappear *)
        if Nul(stok[(k + 1) mod 8]) = false then
         if Nul(stok[(k + 7) mod 8]) = false then
          begin
           alea := myrand(2);
           if alea = 1 then
            enleve_S(indic, stok[(k + 1) mod 8], 1)
           else
            enleve_S(indic, stok[(k + 7) mod 8], 1)
          end
         else
          enleve_S(indic, stok[(k + 1) mod 8], 1)
        else
         enleve_S(indic, stok[(k + 7) mod 8], 1)
       end;
     end;
    indic := indic^.d;
   end;
 end;

 (***************************************************************)
 (*___________PROCEDURES  DE RELIAISON__________________________*)
 (***************************************************************)
 type
  Nbr_pair = 0..12;
  Tab_pair = array[1..2, 1..12] of nbr_vois;
  Tab_bool = array[1..2, 0..7] of boolean;

 procedure corel (X, Y: Coord_Abs; var U: Coord_Rel);
  (*U :coordonnes relatives de Y par rapport a X*)
 (*U: relatives coordivates of Y compared to X*)
 begin
  U[1] := Y[1] - X[1];
  U[2] := Y[2] - X[2];
 end;

 procedure Nv_paire (
    var liable: Nbr_pair;
    i, j: Nbr_Vois;
    var stokpair: Tab_pair);
  (*  rajoute une paire de liable en coordonees ttvois  *)
  (*  add a pair that can be bounded in stock with the
      coordinates of the array ttvois  *)
 begin
  liable := liable + 1;
  stokpair[1, liable] := i;
  stokpair[2, liable] := j;
 end;

 procedure correlien (X: Coord_Abs; var U: Coord_Rel);
  (* renvoie,pour un maillon avec une liaison,la coordonees
     relative U de la liaison*)

  (* gives, for a maillon with a bound, the relative coordinate
     U of the bound *)

  var
   i: integer;
 begin
  i := 0;
  while Esp[X[1], X[2]].lien^[tt_vois[i][1], tt_vois[i][2]] = false do
   i := i + 1;
  U := tt_vois[i];
 end;

 function distance (U, V: Coord_Rel): real;
 begin
  distance := sqr(U[1] - V[1]) + sqr(U[2] - V[2]);
 end;

 function liabl (indic: ptnoeud; i, j: Nbr_Vois; stok: Tab_Vois): boolean;
  (* regarde si les voisins relatifs en coordordonees tt_vois i et j
     de l'element sur laquel point indic sont liables entre eux *)
  (* look if the  neighbours in relatives coordinates i and j in
     stock, of the element indic could be bounded together *)

  var
   X, Y: Coord_Abs;
   U, V, U1, V1: Coord_Rel;
 begin
  X[1] := indic^.coo[1] + stok[i][1];
  X[2] := indic^.coo[2] + stok[i][2];
  Y[1] := indic^.coo[1] + stok[j][1];
  Y[2] := indic^.coo[2] + stok[j][2];
  if (Esp[X[1], X[2]].el = 3) and (Esp[Y[1], Y[2]].el = 3) then
   liabl := true
  else if (Esp[X[1], X[2]].el = 3) then
   begin
    corel(Y, X, U);
    correlien(Y, V);
    if (distance(U, V) > 1) then
     liabl := true
    else
     liabl := false;
   end
  else if (Esp[Y[1], Y[2]].el = 3) then
   begin
    corel(X, Y, U);
    correlien(X, V);
    if (distance(U, V) > 1) then
     liabl := true
    else
     liabl := false;
   end
  else
   begin
    corel(X, Y, U);
    correlien(X, V);
    corel(Y, X, U1);
    correlien(Y, V1);
    if (distance(U, V) > 1) and (distance(U1, V1) > 1) then
     liabl := true
    else
     liabl := false;
   end;
 end;

  procedure list_liable (
    indic: ptnoeud;
    stok: Tab_Vois;
    var liable: Nbr_Pair;
    var stokpair: Tab_Pair);
  (* a partir des voisins liables (stok) selectionne les paires
     de voisins liables 2 a 2 (stokpair)*)
  (* from the neighbours that can be linked (stock), select the
     pairs of neighbours that can be linked together (stockpair) *)

  var
   i: Nbr_Vois;
   k: integer;
 begin
  liable := 0;
  for k := 1 to 12 do
   begin
    stokpair[1, k] := 0;
    stokpair[2, k] := 0;
   end;
  for i := 0 to 7 do
   begin
    if Nul(stok[i]) = false then
     begin
      if (Nul(stok[(i + 1) mod 8]) = false)
       and liabl(indic, i, (i + 1) mod 8, stok) = true
       then
        Nv_paire(liable, i, (i + 1) mod 8, stokpair);
      if i mod 2 = 0 then    (*pas coin*)
       if (Nul(stok[(i + 2) mod 8]) = false)
        and liabl(indic, i, (i + 2) mod 8, stok) = true
        then
         Nv_paire(liable, i, (i + 2) mod 8, stokpair);
     end;
   end;
 end;

  procedure changeT (
    indic: ptnoeud;
    var T: tab_bool;
    stokpair: Tab_pair;
    i, j: integer);
  var
   X: Coord_Abs;
 begin
  T[1, stokpair[i, j]] := false;
  X[1] := indic^.coo[1] + tt_vois[stokpair[i, j]][1];
  X[2] := indic^.coo[2] + tt_vois[stokpair[i, j]][2];
  if Esp[X[1], X[2]].el = 3 then
   T[2, stokpair[i, j]] := true;
 end;

  procedure maxlien (
    indic: ptnoeud;
    stokpair: Tab_Pair;
    liable: Nbr_Pair;
    var maxpair: Nbr_Pair;
    var pair_max: Tab_Pair);

 (* stokpair donne (les paires liables) il y en a liable.
    selectionne un ss ens max de pairs liables (Pair_max) il y en
    a maxpair*)

 (* stockpair gives the pairs that can be linked, their number is
    liable then select a maximal subset of thoses pairs
    (Pair-max), their number is maxpair*)

  var
   i, j, k: integer;
   T: tab_bool;
   mpair: integer;
   pair_m: Tab_pair;
 begin
  for i := 0 to 7 do
   begin
    T[1, i] := true;
    T[2, i] := false;
   end;
  maxpair := 0;
  for i := 1 to 12 do
   begin
    pair_max[1, i] := 0;
    pair_max[2, i] := 0;
   end;
  for i := 1 to liable do
   begin
    changeT(indic, T, stokpair, 1, i);
    changeT(indic, T, stokpair, 2, i);
    for j := 1 to 12 do
     begin
      pair_m[1, j] := 0;
      pair_m[2, j] := 0;
     end;
    pair_m[1, 1] := stokpair[1, i];
    pair_m[2, 1] := stokpair[2, i];
    mpair := 1;
    for j := i + 1 to liable do
     begin
     if ((T[1, stokpair[1, j]] = true) or (T[2, stokpair[1, j]] = true))
       and ((T[1, stokpair[2, j]] = true) or (T[2, stokpair[2, j]] = true))
       then
       begin
        mpair := mpair + 1;
        for k := 1 to 2 do
         begin
          pair_m[k, mpair] := stokpair[k, j];
          if T[2, stokpair[k, j]] = true then
           T[2, stokpair[k, j]] := false
          else
           changeT(indic, T, stokpair, k, j);
         end;
       end;
     end;
    if i = 1 then
     begin
      maxpair := mpair;
      Pair_max := Pair_m;
     end
    else if mpair > maxpair then
     begin
      maxpair := mpair;
      Pair_max := Pair_m
     end;
   end;
 end;

 procedure lie (X, Y: Coord_Abs);
  (* fait la liaison de X a Y et met a jour les listes *)
  (* make the bound between X and Y and refresh the lists -
     lier means to bound *)
  var
   U: Coord_Rel;
   mark: ptnoeud;
 begin
  corel(X, Y, U);
  if Esp[X[1], X[2]].el = 3 then
   init(Esp[X[1], X[2]].lien);
  Esp[X[1], X[2]].lien^[U[1], U[2]] := true;
  recherche(mark, X);
  Esp[X[1], X[2]].el := Esp[X[1], X[2]].el + 1;
  echange(mark, head[Esp[X[1], X[2]].el]);
 end;

  procedure lie_pair (
    indic: ptnoeud;
    maxpair: Nbr_Pair;
    pair_max: Tab_Pair);

  (* assure la lison entr ttes les paires selectionnees *)
  (* make the link for all the selected pairs *)

    var
   i: integer;
   X, Y: Coord_Abs;
 begin
  for i := 1 to maxpair do
   begin
    X[1] := indic^.coo[1] + tt_vois[pair_max[1, i]][1];
    X[2] := indic^.coo[2] + tt_vois[pair_max[1, i]][2];
    Y[1] := indic^.coo[1] + tt_vois[pair_max[2, i]][1];
    Y[2] := indic^.coo[2] + tt_vois[pair_max[2, i]][2];
    lie(X, Y);
    lie(Y, X);
   end;
 end;


 procedure relie (indic: ptnoeud);
  var
   i: integer;
   m: Nbr_vois;
   stok: Tab_Vois;
   liable, maxpair: Nbr_Pair;
   stokpair, pair_max: Tab_Pair;
 begin
  m := 0;
  for i := 0 to 7 do
   begin
    stok[i][1] := 0;
    stok[i][2] := 0;
   end;
  GrWrite('cher ele 1 ');
  cher_ele(indic^.coo, 4, stok, m);
  list_liable(indic, stok, liable, stokpair);
  maxlien(indic, stokpair, liable, maxpair, pair_max);
  lie_pair(indic, maxpair, pair_max);
  m := 0;
  for i := 0 to 7 do
   begin
    stok[i][1] := 0;
    stok[i, 2] := 0;
   end;
  GrWrite('cher ele 2 ');
  cher_ele(indic^.coo, 4, stok, m);
  GrWriteln(' cher ele 3');
  cher_ele(indic^.coo, 3, stok, m);
  list_liable(indic, stok, liable, stokpair);
  maxlien(indic, stokpair, liable, maxpair, pair_max);
  lie_pair(indic, maxpair, pair_max);
 end;

 (****************************************************************)
 (*___________PROCEDURES  DE DESINTEGRATION______________________*)
 (****************************************************************)
 procedure vois_lie (X: ptnoeud; var stok: Tab_Vois; var m: Nbr_Vois);
  (* stok renvoie les coordonnees relatives des voisins auxquels
     X^ est lie *)

  (* stock gives the relatives coordinates of the neigbours to
     whom X is bounded *)

    var
   i: integer;
 begin
  m := 0;
  for i := 0 to 7 do
   begin
    stok[i][1] := 0;
    stok[i, 2] := 0;
   end;
  for i := 0 to 7 do
   begin
    if Esp[X^.coo[1], X^.coo[2]].lien^[tt_vois[i][1], tt_vois[i][2]] = true
     then
      Nv_vois(m, tt_vois[i], stok);
   end;
 end;

 procedure deli_vois (var ptr, X: ptnoeud);
  (* met a jour les voisins lies au desintegre: X pointe sur les
     coordonnees du desintegre. ptr est le repere dans la liste*)
  (* refresh the neighbours  bounded to the desintegred X *)

  var
   stok: Tab_Vois;
   m: Nbr_vois;
   Y, indic: ptnoeud;
   i: integer;
 begin
  new(Y);
  vois_lie(X, stok, m);
  for i := 0 to (m - 1) do
   begin
    Y^.coo[1] := X^.coo[1] + stok[i][1];
    Y^.coo[2] := X^.coo[2] + stok[i][2];

    if Esp[Y^.coo[1], Y^.coo[2]].el = 4 then

        (* maillon lie une fois *)
        (* maillon boubded one time*)

     begin
      dispose(Esp[Y^.coo[1], Y^.coo[2]].lien);
      GrWrite('rech1');
      recherche(indic, Y^.coo);
      Esp[Y^.coo[1], Y^.coo[2]].el := 3;
      if indic = ptr then
       begin
        GrWrite('SAUVE');
        ptr := ptr^.g;
       end;
      echange(indic, head[3]);
     end


        (* maillon lie deux fois *)
        (* maillon boubded two times *)
    else
     begin
      Esp[Y^.coo[1], Y^.coo[2]].lien^[-stok[i][1], -stok[i][2]] := false;
      GrWrite('rech2');
      recherche(indic, Y^.coo);
      Esp[Y^.coo[1], Y^.coo[2]].el := 4;
      if indic = ptr then
       begin
        GrWrite('SAUVE');
        ptr := ptr^.g;
       end;
      echange(indic, head[4]);
     end;
   end;
  dispose(Y);
 end;

 procedure trou_proch (X: ptnoeud);
  (* recherche le trou le plus proche de X et le transforme en substrat *)
  (* look for the nearest whole of X and transform it into substrat *)
  var
   indic, indic_trou: ptnoeud;
   d, dmin: real;
 begin
  indic := head[1].d;
  indic_trou := indic;
    (*rech du trou le plus  proche*)
  dmin := sqr(indic^.coo[1] - X^.coo[1]) + sqr(indic^.coo[2] - X^.coo[2]);
  while indic <> nil do
   begin
    d := sqr(indic^.coo[1] - X^.coo[1]) + sqr(indic^.coo[2] - X^.coo[2]);
    if d < dmin then
     begin
      indic_trou := indic;
      dmin := d;
     end;
    indic := indic^.d;
   end;
    (* transformation du trou en substrat *)
    (* transformation of the whole in substrat *)
  Esp[indic_trou^.coo[1], indic_trou^.coo[2]].el := 0;
  GrWrite('      trou ');
  Str(indic_trou^.coo[1], s);
  GrWrite(s);
  GrWrite(' ');
  Str(indic_trou^.coo[2], s);
  GrWriteLn(s);
  (* writeln('      trou ', indic_trou^.coo[1], '  ', indic_trou^.coo[2]);*)
  supprime(indic_trou);
 end;

 procedure Desintegre;
  var
   i: integer;
   indic, indic1, indic2, indic_trou: ptnoeud;
   alea, d, dmin: extended;
 begin
    for i := 3 to 5 do
      (* examine tout les maillons, lies ou non*)
      (* examination of all substrates, bounded or not*)
   begin
    indic := @head[i];
    while indic^.d <> nil do
     begin
      (* alea := random / 65534 + 0.5;*)
       alea := Random;
       if alea <= Pd then
              (* desintegration avec une probabilite Pd *)
              (* desintegration with probability Ps*)
       begin

        indic1 := indic^.d;
        (* writeln(
         'des  ',
         i : 3,
         Esp[indic1^.coo[1], indic1^.coo[2]].el,
         indic1^.coo[1],
         '  ',
         indic1^.coo[2]); *)
        GrWrite('des ');
        Str(i : 3, s);
        GrWrite(s);
        Str(Esp[indic1^.coo[1], indic1^.coo[2]].el, s);
        GrWrite(s);
        Str(indic1^.coo[1], s);
        GrWrite(s);
        GrWrite(' ');
        Str(indic1^.coo[2], s);
        GrWriteLn(s);
                (* cherch du trou le plus proche du substrat *)
                (* look for the whole the nearest of the substrat *)
        trou_proch(indic1);
                (* met a jour les voisins a qui il est lie:
                   delie puis relie*)
                (* refresh the neighbours to whom he is bounded *)
        if i > 3 then
         begin
          GrWriteLn('on deloie ');
          Deli_Vois(indic, indic1);
          GrWriteLn(' on relie ');
          relie(indic1);
          GrWriteln(' on a relie');
         end;
                (* transformation en substrat *)
        Esp[indic1^.coo[1], indic1^.coo[2]].el := 0;
        supprime(indic1);
       end
      else
       indic := indic^.d;
     end;
   end;
 end;


 (*************************************************************)
 (*____________DETECTION D'UNE UNITE  ________________________*)
 (*************************************************************)

 (*detect if there is a unit*)

  function unite: boolean;
  type
   vect = array[1..100] of coord_abs;
  var
   ptvect: ^vect;
   X, Y: coord_abs;
   i, j, k: integer;
   indic, indic1: ptnoeud;
   boucle, dr: boolean;

 begin
  new(ptvect);
  if head[5].d <> nil then
   begin
    indic := head[5].d;
    dr := false;
    while (indic <> nil) and (dr = false) do
     begin
      boucle := false;
      X := indic^.coo;
      ptvect^[1] := X;
      k := 1;
      i := 0;
      while esp[X[1], X[2]].lien^[tt_vois[i][1], tt_vois[i][2]] = false
       do
       i := i + 1;
      Y := X;

           (***  detect a ring  ****)

      repeat
       Y[1] := Y[1] + tt_vois[i][1];
       Y[2] := Y[2] + tt_vois[i][2];
       k := k + 1;
       ptvect^[k] := Y;
       if esp[Y[1], Y[2]].el = 5 then
        begin
         if (Y[1] = X[1]) and (Y[2] = X[2]) then
          boucle := true
         else
          begin
           j := 0;
           while (esp[Y[1], Y[2]].lien^
             [tt_vois[j][1], tt_vois[j][2]] = false)
            or (j = (i + 4) mod 8)
            do
             j := j + 1;
          end;
         i := j;
        end;
      until (boucle = true) or (Esp[Y[1], Y[2]].el = 4);


   (* detecte si un catalyseur est a l'interieur de la boucle *)
   (* detect if a catalyst is inside the ring *)

      if boucle = true then
       begin
        GrWriteln('BOUCLE');
        indic1 := head[2].d;
        while (indic1 <> nil) and (dr = false) do
         begin
          X := indic1^.coo;
          Y := X;
          if (X[1] <> 1)
           and (X[1] <> n)
           and (X[2] <> 1)
           and (X[2] <> n)
           then
           begin
            repeat
             Y[1] := Y[1] + 1;
             i := 1;
             while (i <> k + 1)
              and ((Y[1] <> ptvect^[i][1]) or (Y[2] <> ptvect^[i][2]))
              do
               i := i + 1;
            until (i < k + 1) or (Y[1] = N);
            if i < k + 1 then
             dr := true;
            if dr = true then
             begin
              Y := X;
              repeat
               Y[1] := Y[1] - 1;
               i := 1;
               while (i <> k + 1)
                and ((Y[1] <> ptvect^[i][1]) or (Y[2] <> ptvect^[i][2]))
                do
                 i := i + 1;
              until (i < k + 1) or (Y[1] = 1);
              if i < k + 1 then
               dr := true
              else
               dr := false;
              if dr = true then
               begin
                Y := X;
                repeat
                 Y[2] := Y[2] + 1;
                 i := 1;
                 while (i <> k + 1)
                  and ((Y[1] <> ptvect^[i][1]) or (Y[2] <> ptvect^[i][2]))
                  do
                   i := i + 1;
                until (i < k + 1) or (Y[2] = n);
                if i < k + 1 then
                 dr := true
                else
                 dr := false;
                if dr = true then
                 begin
                  Y := X;
                  repeat
                   Y[2] := Y[2] - 1;
                   i := 1;
                   while (i <> k + 1)
                    and ((Y[1] <> ptvect^[i][1]) or (Y[2] <> ptvect^[i][2]))
                    do
                     i := i + 1;
                  until (i < k + 1) or (Y[2] = 1);
                  if i < k + 1 then
                   dr := true
                  else
                   dr := false;
                 end;
               end;
             end;
           end;
          indic1 := indic1^.d;
         end;
       end;


      indic := indic^.d;
     end;
   end;
  dispose(ptvect);
  if dr = true then
   unite := true
  else
   unite := false;
 end;


 (***************************************************************)
 (*____________PROGRAMME PRINCIPAL    __________________________*)
 (***************************************************************)
 var
  i, j, k: integer;
  cara: char;

(*  even: EventRecord; *)
  even: Integer;

  c: boolean;

begin
 (* showtext; *)
 (* writeln('debur ok'); *)
 (* GetDateTime(randSeed);*) (* Init random number generator *)

 writeln;
 writeln;
 writeln('FRJ_SAP (FRJ''s Simple Autopoiesis Program) version 1.0');
 writeln('Copyright (C) 1995');
 writeln('  Francois Jullien <frj@ccr.jussieu.fr>');
 writeln('  Barry McMullin   <McMullin@eeng.dcu.ie>');
 writeln;
 writeln('FRJ_SAP comes with ABSOLUTELY NO WARRANTY. This is free software,');
 writeln('and you are welcome to redistribute it under certain conditions.');
 writeln('Read the file LICENSE.TXT for further information.');
 writeln;
 writeln('(Press any key to continue...)');
 writeln;
 Pause;

 Randomize;
 initialise;
 init_dessin;

 for k := 1 to niter do
  begin

   SetLogWindow;
   ConditionalPause;

   (* WriteLn(' iteration ', k : 3, ' '); *)
   GrWrite(' iteration ');
   str(k : 3, s);
   GrWriteLn(s);

   GrWriteLn('mouvement');
   move_vide;
   move_maillon;
   move_cata;


   GrWriteLn('production ');
   production;


   GrWriteLn('destruction ');
   desintegre;

(*    SelectWindow(wpt2);    SetPort ( wpt2 ); *)
   dessin;

   if unite = true then
    begin
     GRWriteLn(' ');
     GRWriteLn('UNITE');
     (* Pause; *)
    end;

(*    SelectWindow(wpt); SetPort ( wpt ); *)
   desunit(k, unite);

  end;

 (*
 repeat
  SystemTask;
  c := GetNextEvent(everyEvent, even);
 until
  (even.what = mouseDown)
    and (FindWindow(even.where, wpt) = inContent); *)
    (* loop forever *)

 Pause;

end.
