(*

    Program: FRJ_SAP
    File:    GLOBAL.PAS
    Copyright (C) 1995
      Francois Jullien <frj@ccr.jussieu.fr>
      Barry McMullin   <mcmullin@eeng.dcu.ie>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 1, or (at your option)
    any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program, in the file LICENSE.TXT; if not, write to
    the Free Software Foundation, Inc., 675 Mass Ave, Cambridge, MA
    02139, USA.

*)

(***************************************************)

(* GLOBAL.PAS: Module defining global variables. *)

unit Global;

interface

 const
    n = 20;     (* size of the squarre space *)
    Pd = 0.05;  (* probability of desintegration *)
    niter = 200;  (* number of iterations *)
 type
  Dim = 0..n;
  (* BMcM: "Normally" absolute co-ordinates will run from
           1..n  - but there seems to be a special case in
           Move_Maillon() where 0 is used as a "null"
           index... *)

  Petit = -2..2;
  Nbr_Vois = 0..8; (*vois is for voisin which means neighbour*)
  Sorte = 0..5;

  (* type of element:
     0:substrat
     1:empty
     2:catalysist
     3:maillon free
     4:maillon bounded one time
     5:maillon bounded two time *)

  Coord_Rel = array[1..2] of Petit;
      (* coordinates relative in the space *)
  Coord_Abs = array[1..2] of Dim;
      (* coordinates absolute in the space *)
  Tab_Vois = array[0..7] of Coord_Rel;


  (* definition of lists: exept for the substrat, we have for
     each type a list of the coordinates in the space of all the
     elements of this type *)

  ptnoeud = ^noeud; (*noeud means node*)
  noeud = record
    coo: Coord_Abs;
    g: ptnoeud;
    d: ptnoeud;
   end;
  tetes = array[1..5] of noeud;
      (* array of lists of elements from 1 to 5 *)


  (* definition of space: when the element is a substrat bounded,
     the direction of the bound(s) are indicated with the array
     of boolean: it's true in the direction of the link*)

  Tab = array[-1..1, -1..1] of boolean;
  Pttab = ^Tab;
  Point = record
    el: Sorte;
    lien: Pttab
   end;
  Espace = array[dim, dim] of point;

 var
(*  wpt, wpt1, wpt2: windowptr; *)
  vois_card: Tab_Vois;
      (* tableau contenant les coordonnees relatives
         des 4 voisins cardinaux  *)
      (* array which containd the relatives coordinates
         of the 4 cardinal neighbour *)
  tt_vois: Tab_Vois;
      (* tableau contenant les coordonnees relatives
         des 8 voisins possibles*)
      (* array which containd the relatives coordinates of the
         8 possible neighbour *)

  esp: Espace;
  head: tetes; (*tete means head*)
  pt: pointer;

  s: string[80]; (* Miscellaneous conversions... *)

implementation

end.

